//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "IrcSocket.h"
#include "packets.h"
#include "opcodes.h"
#include "ircmain.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// By Maverick Added CIrcSocketBaseClass 
// to make it easier to change BaseSocket
CIrcSocket::CIrcSocket(CIrcMain* pIrcMain) : CIrcSocketBaseClass()
{
	m_pIrcMain = pIrcMain;
	m_pProxyLayer = NULL;
}

CIrcSocket::~CIrcSocket()
{
}


#if 0
// By Maverick Added CIrcSocketBaseClass 
// to make it easier to change BaseSocket
BEGIN_MESSAGE_MAP(CIrcSocket, CIrcSocketBaseClass)
END_MESSAGE_MAP()
#endif

void CIrcSocket::Connect()
{
	EMULE_TRY

    // Maverick Connect to proxy
	RemoveAllLayers();

	// ProxyInitialisation
	ProxySettings settings = theApp.glob_prefs->GetProxy();

	if (settings.UseProxy && settings.type!=PROXYTYPE_NOPROXY)
	{
		m_pProxyLayer=new CAsyncProxySocketLayer;
		switch (settings.type)
		{
			case PROXYTYPE_SOCKS4:
				m_pProxyLayer->SetProxy(PROXYTYPE_SOCKS4,settings.name,settings.port);
				break;
			case PROXYTYPE_SOCKS4A:
				m_pProxyLayer->SetProxy(PROXYTYPE_SOCKS4A,settings.name,settings.port);
				break;
			case PROXYTYPE_SOCKS5:
				if (settings.EnablePassword)
					m_pProxyLayer->SetProxy(PROXYTYPE_SOCKS5,settings.name, settings.port, settings.user, settings.password);
				else
					m_pProxyLayer->SetProxy(PROXYTYPE_SOCKS5,settings.name,settings.port);
				break;
			case PROXYTYPE_HTTP11:
				if (settings.EnablePassword)
					m_pProxyLayer->SetProxy(PROXYTYPE_HTTP11,settings.name,settings.port, settings.user, settings.password);
				else
					m_pProxyLayer->SetProxy(PROXYTYPE_HTTP11,settings.name,settings.port);
				break;
			default: ASSERT(FALSE);
		}
		AddLayer(m_pProxyLayer);
	}

	// Connection Initialisation
	CIrcSocketBaseClass::Create();
	AsyncSelect(FD_READ | FD_WRITE | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE);


	// By Maverick Added CIrcSocketBaseClass 
	// to make it easier to change BaseSocket
	CIrcSocketBaseClass::Connect(theApp.glob_prefs->GetIRCServer(), 6667);
	// END Maverick

	EMULE_CATCH
}

void CIrcSocket::OnReceive(int nErrorCode) 
{
    //Maverick Changes for PROXY connection
	CIrcSocketBaseClass::OnReceive(nErrorCode);
	if (nErrorCode == 0)
	{
		char buffer[256];
		int length = Receive(buffer, sizeof(buffer)-1);	
		if (length > 0)
		{
			buffer[length] = '\0';
	                theApp.downloadqueue->AddDownDataOverheadOther(length*8);
			m_pIrcMain->PreParseMessage( buffer );
		}
	}
    // END Maverick
}

void CIrcSocket::OnConnect(int nErrorCode)
{
	m_pIrcMain->SetConnectStatus(true);
	m_pIrcMain->SendLogin();
}
void CIrcSocket::OnClose(int nErrorCode)
{
	// By Maverick Added CIrcSocketBaseClass 
	// to make it easier to change BaseSocket
	CIrcSocketBaseClass::Close();
	RemoveAllLayers();
	m_pIrcMain->SetConnectStatus(false);
}

int CIrcSocket::SendString(CString message){
	message += "\r\n";
	int size = message.GetLength();
	theApp.uploadqueue->AddUpDataOverheadOther(size*8);
	return Send(message, size);
}

// By Maverick
//Resets layer chain.
void CIrcSocket::RemoveAllLayers()
{
	EMULE_TRY
	CIrcSocketBaseClass::RemoveAllLayers();
	
	// ProxyLayer Destruction
	if (m_pProxyLayer) 
	{
		delete m_pProxyLayer;
		m_pProxyLayer = NULL;
	}
	EMULE_CATCH
}

int CIrcSocket::OnLayerCallback(const CAsyncSocketExLayer *pLayer, int nType, int nParam1, int nParam2)
{
	EMULE_TRY
	ASSERT(pLayer);
	if (nType==LAYERCALLBACK_STATECHANGE)
	{
		CString logline;
		if (pLayer==m_pProxyLayer)
		{
			//logline.Format(_T("ProxyLayer changed state from %d to %d"), nParam2, nParam1);
			//AddLogLine(false,logline);
		}else
			//logline.Format(_T("Layer @ %d changed state from %d to %d"), pLayer, nParam2, nParam1);
			//AddLogLine(false,logline);
		return 1;
	}
	else if (nType==LAYERCALLBACK_LAYERSPECIFIC)
	{
		if (pLayer==m_pProxyLayer)
		{
			switch (nParam1)
			{
			case PROXYERROR_NOCONN:
				AddLogLine(false,GetResString(IDS_ERRORMSG_PROXY_NOCONN));
				break;
			case PROXYERROR_REQUESTFAILED:
				AddLogLine(false,GetResString(IDS_ERRORMSG_PROXY_REQUESTFAILED));
				break;
			case PROXYERROR_AUTHTYPEUNKNOWN:
				AddLogLine(false,GetResString(IDS_ERRORMSG_PROXY_AUTHTYPEUNKNOWN));
				break;
			case PROXYERROR_AUTHFAILED:
				AddLogLine(false,GetResString(IDS_ERRORMSG_PROXY_AUTHFAILED));
				break;
			case PROXYERROR_AUTHNOLOGON:
				AddLogLine(false,GetResString(IDS_ERRORMSG_PROXY_AUTHNOLOGON));
				break;
			case PROXYERROR_CANTRESOLVEHOST:
				AddLogLine(false,GetResString(IDS_ERRORMSG_PROXY_CANTRESOLVEHOST));
				break;
			default:
				AddLogLine(false,__FILE__, __LINE__, _T("Unknown proxy error") );
			}
		}
	}
	EMULE_CATCH
	return 1;
}

